/**
 * MPQAAThread.java: MPQ Audio Analyzer.
 *            Copyright (C) 2013 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2013-01-27 donfede@casagrau.org
 * $Id:#
 */

package org.casagrau.mpq.audio_service;

import android.content.Context;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;

import com.ringdroid.soundfile.CheapSoundFile;

public class MPQAAThread extends MPQAudioWorkerThread {
	private static final String LOGTAG = "MPQAAThread";

	
	/** 
	 * Constructor for the worker thread.
	 * {@link #MPQAudioWorkerThread}  
	 */
	public MPQAAThread(Handler parentHandler, String filename, long filepos, int state, Context context) {
		super(parentHandler, filename, filepos, state, context);
	}
	

	/**
	 * Perform 'basic' audio analysis.
	 * 
	 */
	public void run() {
		Log.d(LOGTAG, "runAA() 00 filename: " + mFilename + " len: " + mFilelen + " noise " + mFilenoise);

		// Extract statistics from a CheapSoundFile object.
		CheapSoundFile lcheapSound = null;
    	long lfileMS = 0;
    	long lnoiseMS = 0;
		int lnoiseFrames = 0;
    	int lchannelCount = 0;
		int ltotalFrames = 0;
		int lframeBytes = 0;
    	int lsampleBit = 0; // sample bitrate
		int lsampleRate = 0; 
		try {
			String lFileName = Environment.getExternalStorageDirectory().getAbsolutePath() + mFilename;
			lcheapSound = CheapSoundFile.create(lFileName, null, mServiceContext);
			lnoiseFrames = lcheapSound.getNumNoiseFrames();
			ltotalFrames = lcheapSound.getNumFrames();
			lframeBytes = lcheapSound.getFrameBytes();
			
			lsampleRate = lcheapSound.getSampleRate();
			lsampleBit = lcheapSound.getBitRate();			
			lchannelCount = lcheapSound.getChannels();
			
//			int lsamplesPFrame = lcheapSound.getSamplesPerFrame(); // FIXME: consider tossing this...			
//			Log.w(LOGTAG, "runAA() 32 tFrame: " + ltotalFrames + " nFrames: " + lnoiseFrames + " frameBytes: " + lframeBytes + " sampPFrame: " + lsamplesPFrame + " chan: " + lchannelCount + " samplebit: " + lsampleBit + " sampleRate|freq: " + lsampleRate);
			lfileMS = ((long)ltotalFrames * lframeBytes * 1000) / lchannelCount / (lsampleBit / 8) / lsampleRate;
			lnoiseMS = ((long)lnoiseFrames * lframeBytes * 1000) / lchannelCount / (lsampleBit / 8) / lsampleRate;  		
		} catch (final Exception e) {
			Log.e(LOGTAG, "runAA() Exception: " + e.getLocalizedMessage() );
			e.printStackTrace();
			// FIXME: do we handle this exception differently than returning early (without sending AA results)??
			return;
		}
		
		// Send updates with AA results.  Report results back (from child thread to parent service; from service to SessionDetailFragment to DB).    	
    	sendPMsgAA(mFilename, lfileMS, lnoiseMS);
		Log.d(LOGTAG, "run() 99 filename: " + mFilename + " len: " + lfileMS + " noise " + lnoiseMS);
		return;
	} // END run()
		
} // END class MPQAAThread
