/**
 * MPQAudioWorkerRecThread.java: Recording worker thread class.
 *            Copyright (C) 2012,2013 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-12-04 donfede@casagrau.org
 * credit to: Reto Meier, EMEA Android Developer Advocate, Google UK; for a record and playback example -- http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
 *			  Stjepan Rajko, urbanSTEW -- (GPL) RehearsalAssistant/RehearsalAudioRecorder.java
 *			  cristi (Cristian Baita) -- http://indyvision.net/2010/02/android-threads-tutorial-part-3/
 *			  Ringdroid, with a primary developer of Dominic Mazzoni, a Google employee and also author of Audacity. --  http://code.google.com/p/ringdroid/
 * $Id: MPQAudioWorkerRecThread.java,v 1.8 2013-02-19 22:42:25 donfede Exp $
 */

package org.casagrau.mpq.audio_service;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.casagrau.mpq.mpq_android.R;

import android.content.Context;
import android.content.SharedPreferences;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.Environment;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.util.Log;



public class MPQAudioWorkerRecThread extends MPQAudioWorkerThread {
	private static final String LOGTAG = "MPQAudioWorkerRecThread";
	
	/** 
	 * Constructor for the worker thread.
	 * {@link #MPQAudioWorkerThread}  
	 */
	public MPQAudioWorkerRecThread(Handler parentHandler, String filename, long filepos, int state, Context context) {
		super(parentHandler, filename, filepos, state, context);
	}
		
	
	/**
	 * Record audio to a wave file.
	 * This code is meant to be started from a thread, and will run until 
	 * a max time limit is hit (recording or pause), at which point the file will be closed.
	 * Credit to: Reto Meier, EMEA Android Developer Advocate, Google UK -- http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
	 * 	          Ringdroid, with a primary developer of Dominic Mazzoni, a Google employee and also author of Audacity. --  http://code.google.com/p/ringdroid/
	 * @param intent with the action and extra content to process. 
	 * 					Extra ARG_FILENAME is checked for to open.
	 * @throws IllegalStateException if there are errors writing the file
	 * @throws IOException if there are errors opening the audio device
	 */
	public void run() { 
		// FIXME: get these values from preferences
		int lfrequency = 11025; 
		int lchannelConfiguration = AudioFormat.CHANNEL_IN_MONO;
		int laudioEncoding = AudioFormat.ENCODING_PCM_16BIT; 
	    Log.d(LOGTAG, "runREC() 00 freq: " + lfrequency + " lchan: " + lchannelConfiguration + " lenc: " + laudioEncoding);

    	short lchannelCount;
    	if (AudioFormat.CHANNEL_IN_MONO == lchannelConfiguration) lchannelCount = 1;
    	else if (AudioFormat.CHANNEL_IN_STEREO == lchannelConfiguration) lchannelCount = 2;
    	else throw new IllegalStateException("unsuported channel configuration: " + lchannelConfiguration);

    	short lsampleBit = 0; // sample bitrate
    	if (AudioFormat.ENCODING_PCM_16BIT == laudioEncoding) lsampleBit = 16;
    	else if (AudioFormat.ENCODING_PCM_8BIT == laudioEncoding) lsampleBit = 8;
    	else throw new IllegalStateException("unsuported sample bitrate: " + lsampleBit);
    	
    	int lframeSize = (lsampleBit/8)*lchannelCount;
        long lbyteRate = lfrequency * (lsampleBit/8) * lchannelCount;
		Log.d(LOGTAG, "runREC() 06  mStateThread: " + mStateThread);
		
		File lFile; // Local file handle;

		
		// Get audio directory from preferences.
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(mServiceContext);
		String laudioDirPref = sharedPref.getString(mServiceContext.getString(R.string.pref_audiodirectory_key), mServiceContext.getString(R.string.pref_audiodirectory_default));

		
	    // Confirm parent directory exists.
		lFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + laudioDirPref);
	    Log.d(LOGTAG, "runREC() 10 file basedir: " + lFile + " using audioDir: " + laudioDirPref);

	    if (!lFile.isDirectory()) {
		    Log.d(LOGTAG, "runREC() 12 DIR not directory - del");
	    	lFile.delete();
	    }
	    if (!lFile.exists() ) {
		    Log.d(LOGTAG, "runREC() 14 DIR no exists - mkdir");
	    	lFile.mkdirs();
	    }
	    	
		lFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + mFilename);
	    Log.d(LOGTAG, "runREC() 20 FILE file: " + lFile);
		// Delete any previous recording.
		if (lFile.exists())
			lFile.delete();

		// Create the new file.
		try {
			lFile.createNewFile();
		} catch (IOException e) {
			throw new IllegalStateException("Failed to create " + lFile.toString());
		}

		try {
			// Create a new AudioRecord object to record the audio.
			int lbufferSize = AudioRecord.getMinBufferSize(lfrequency, lchannelConfiguration, laudioEncoding);
			lbufferSize = 2 * lbufferSize;
			byte[] lbuffer = new byte[lbufferSize];
	        Log.d(LOGTAG, "runREC() 40  bufSize(2xmin): " + lbufferSize);
			AudioRecord laudioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC,
					lfrequency, lchannelConfiguration,
					laudioEncoding, lbufferSize);

			if (laudioRecord.getState() != AudioRecord.STATE_INITIALIZED)
				throw new IOException("runREC() AudioRecord initialization failed");
//			audioRecord.setRecordPositionUpdateListener(updateListener);
//			audioRecord.setPositionNotificationPeriod(framePeriod);
			
			
			// Prepare the WAVE file header. -- credit: Dominic Mazzoni, RingDroid CheapWAV
			byte[] lheader = new byte[44];
			lheader[0] = 'R';  // RIFF/WAVE header
			lheader[1] = 'I';
			lheader[2] = 'F';
			lheader[3] = 'F';
			lheader[4] = 0;
			lheader[5] = 0;
			lheader[6] = 0;
			lheader[7] = 0;
			lheader[8] = 'W';
			lheader[9] = 'A';
			lheader[10] = 'V';
			lheader[11] = 'E';
			lheader[12] = 'f';  // 'fmt ' chunk
			lheader[13] = 'm';
			lheader[14] = 't';
			lheader[15] = ' ';
			lheader[16] = 16;  // 4 bytes: size of 'fmt ' chunk
			lheader[17] = 0;
			lheader[18] = 0;
			lheader[19] = 0;
			lheader[20] = 1;  // format = 1
			lheader[21] = 0;
			lheader[22] = (byte) lchannelCount;
			lheader[23] = 0;
			lheader[24] = (byte) (lfrequency & 0xff);
			lheader[25] = (byte) ((lfrequency >> 8) & 0xff);
			lheader[26] = (byte) ((lfrequency >> 16) & 0xff);
			lheader[27] = (byte) ((lfrequency >> 24) & 0xff);
			lheader[28] = (byte) (lbyteRate & 0xff);
			lheader[29] = (byte) ((lbyteRate >> 8) & 0xff);
			lheader[30] = (byte) ((lbyteRate >> 16) & 0xff);
			lheader[31] = (byte) ((lbyteRate >> 24) & 0xff);
			lheader[32] = (byte) (2 * lbyteRate);  // block align
			lheader[33] = 0;
			lheader[34] = (byte) lsampleBit;  // bits per sample
			lheader[35] = 0;
			lheader[36] = 'd';
			lheader[37] = 'a';
			lheader[38] = 't';
			lheader[39] = 'a';
			lheader[40] = 0;
			lheader[41] = 0;
			lheader[42] = 0;
			lheader[43] = 0;

			// Write initial WAVE file headers. -- credit: Stjepan Rajko
			mFileHandle = new RandomAccessFile(lFile, "rw");	    	
			mFileHandle.setLength(0); // Set file length to 0, to prevent unexpected behavior in case the file already existed

			mFileHandle.write(lheader, 0, 44);
			
			laudioRecord.startRecording();
			long lnowTime = System.currentTimeMillis();
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			String lstartTimeStr = formatter.format(new Date(lnowTime));
			Log.d(LOGTAG, "runREC() 50  str start: " + lstartTimeStr);

			long lpayloadSize = 0;// Size of the audio recorded, used later in wave headers.
			mFilepos = 0;
			sendPMsgPos(mFilepos, false);
			int lbufferReadSize;	// Variable to store the size read into the buffer.
			// Recording loop, while state is to record.
			while ( (MPQAudioService.FLG_SRV_STATE_REC == mStateThread) || 
					(MPQAudioService.FLG_SRV_STATE_PAUSE_REC == mStateThread) ) {
				lbufferReadSize = laudioRecord.read(lbuffer, 0, lbufferSize);
				
				if (MPQAudioService.FLG_SRV_STATE_REC == mStateThread) {
	    			lpayloadSize = lpayloadSize + lbufferReadSize;
	    			mFileHandle.write(lbuffer, 0, lbufferReadSize);
	    			mFilepos = cursorFrameToMS(lpayloadSize, lframeSize, lfrequency);
	    			sendPMsgPos(mFilepos, false);
	        		Log.d(LOGTAG, "runREC() 72      IN while loop recording buffer.len: " + lbuffer.length + " payloadSize: " + lpayloadSize );
	        		Log.d(LOGTAG, "runREC() 74               loop recording filepos: " +  + mFileHandle.getFilePointer() );
				} // else discard recorded buffer 

				lnowTime = System.currentTimeMillis();
	    		Log.d(LOGTAG, "runREC() 78      IN while loop mState: " + mStateThread + " bufsize: " + lbuffer.length + " nowTime:" + formatter.format(new Date(lnowTime)) );
	    		if (MPQAudioService.FLG_SRV_STATE_PAUSE_REC == mStateThread) {
	    			try {
	    				Thread.sleep(AUDIO_PAUSE_MS);
	    			} catch (Exception e) {
	    				e.getLocalizedMessage();
	    			}
	        		Log.d(LOGTAG, "runREC() 79      IN while PAUSE WAIT mState: " + mStateThread + " nowTime:" + formatter.format(new Date(lnowTime)) );
				} // END if pause wait
			} // END while recording loop
			Log.d(LOGTAG, "runREC() 80  OUT of while loop to record now time:" + formatter.format(new Date(lnowTime)) );
			laudioRecord.stop();
			laudioRecord.release();

			mFileHandle.getFD().sync();
			// Cleanup and update WAV headers with size
			try	{
				mFileHandle.seek(4); // Write size to RIFF header
				mFileHandle.writeInt(Integer.reverseBytes(36+(int)lpayloadSize)); // FIXME: *** evil hack - casting payloadSize to int -- need to review upper limit (set by WAVE format) and adapt accordingly ...
				mFileHandle.seek(40); // Write size to Subchunk2Size field
				mFileHandle.writeInt(Integer.reverseBytes((int)lpayloadSize)); // FIXME: *** evil hack - casting payloadSize to int -- need to review upper limit (set by WAVE format) and adapt accordingly ...
				mFileHandle.close();
			} catch(IOException e) {
				Log.e(LOGTAG, "I/O exception occured while closing output file");
				//FIXME: likely need to implement an error state...				state = State.ERROR;
				e.getLocalizedMessage();
			}
			Log.d(LOGTAG, "runREC() 90  WAVE headers updated - payloadSize: " + lpayloadSize + " mStateThread: " + mStateThread);
			double lfrequencyMS = lfrequency / 1000.0; // cycles per millisecond
	    	long lframeCount = lpayloadSize / lframeSize;    	
	    	long lfileMS = (long) (lframeCount / lfrequencyMS); // FIXME: review/re-think if this float division is necessary...
	    	Log.d(LOGTAG, "runREC() 94  fileMS: " + lfileMS + " frameCount: " + lframeCount + " lfreq: " + lfrequency + " lfreqMS: " + lfrequencyMS);

			// Set state to stopped, and send Intents to open and analyze audio on the file.
			mFilepos = 0;
			sendPMsgPos(mFilepos, true);
			if (MPQAudioService.FLG_SRV_STATE_STOP != mStateThread) {
	    		Log.d(LOGTAG, "runREC() 94 send startService intent STOP");
	    		sendSvcIntentStop();
			}
    		Log.d(LOGTAG, "runREC() 97 send startService intent OPEN");
    		sendSvcIntentOpen();
    		
    		Log.d(LOGTAG, "runREC() 98 send startService intent AA to analyze audio");
    		sendSvcIntentAA();

		} catch (Exception e) {
			Log.e(LOGTAG,"runREC() Recording Failed e:" + e.getLocalizedMessage());
		}
		Log.d(LOGTAG, "runREC() 99  ");
	} // END runREC()


    
} // END class MPQAudioWorkerRecThread
